#!/bin/sh
exec /home/omry/bin/scala -classpath -savecompiled -cp "lib/*" $0 $@
!#

import scala.io.Source
import scala.xml._
import scala.actors.Actor
import sys.process._
import java.io._
import com.sun.syndication.feed._
import com.sun.syndication.feed.synd._
import com.sun.syndication.io._

object Conf
{
	val YOUTUBE_CHANNEL_URL = "http://gdata.youtube.com/feeds/api/users/%s/uploads"
	val YOUTUBE_PROFILE_URL = "http://gdata.youtube.com/feeds/api/users/%s"
	val BIN_DIR = "/home/omry/youtube-rss-2/bin"
	val DOWNLOAD_DIR = "/home/omry/youtube-rss-2/download"
	val RSS_WEB_DIR="/home/omry/www/youtube-rss.firefang.net"
	val RSS_BASE_URL="http://youtube-rss.firefang.net"
	val DELETE_OLDER=14
}

/**
TODO:
	command line options for:
		delete old files
		update feed files
		download and encode
		server mode
	external configuration file
	possibly add other data to feed
*/
object YoutubeRSS 
{
	val downloader = new Downloader

    def main(args: Array[String]) 
	{
		println("Starting YoutubeRSS")
		deleteOldFiles
		regenerateFeeds
		Source.fromFile("channels").getLines.foreach(x => process(x trim) )
		downloader ! Stop
	}

	def process(channel : String)
	{
		println("Processing channel " + channel)
		updateFeed(channel)
		val url = Conf.YOUTUBE_CHANNEL_URL.format(channel)
		val rss = Source.fromURL(url).mkString
		val xml = XML.loadString(rss)
		val entries = xml\"entry";
		entries foreach(download(_,channel))
		println("Done processing " + channel)

	}

	def regenerateFeeds
	{
		new File(Conf.RSS_WEB_DIR).listFiles.
			filter(_.isDirectory()).
			foreach(file => updateFeed(dirChannel(file)))
	}

	def download(entry : NodeSeq, channel : String)
	{
		downloader ! new Download(entry, channel)
	}

	def dirChannel(f : String) = 
	{
		val matcher = """.*/(.*)""".r
		val matcher(chan) = f
		chan
	}

	def deleteOldFiles
	{
		val files = ("find %s -mtime +%d".format(Conf.DOWNLOAD_DIR,Conf.DELETE_OLDER)!!)
		files split "\n" filter(_ != "") foreach(line => 
		{
			val f = line.substring(Conf.DOWNLOAD_DIR.length+1)
			println("Deleting " + f)
			new File(line).delete
			new File(Conf.RSS_WEB_DIR + "/" +  f) delete
			val matcher = """(.*)/.*""".r
			val matcher(channel) = f
			updateFeed(channel)
		})
	}
}

def updateFeed(channel : String)
{
	val rss = Source.fromURL(Conf.YOUTUBE_PROFILE_URL.format(channel)).mkString
	val xml = XML.loadString(rss)
	val title =  ((xml\"title") text)
	println("Updating feed for %s (%s)".format(title, channel))
	val feed = new SyndFeedImpl();
	feed.setEncoding("UTF-8");
	feed.setFeedType("rss_2.0");
	feed.setTitle(title);
	feed.setLink(Conf.RSS_BASE_URL+"/"+channel);
	feed.setDescription(title);
	var entries = new java.util.ArrayList[SyndEntryImpl]();
	feed.setEntries(entries);

	var dir = Conf.RSS_WEB_DIR + "/" + channel
	val formatter = new java.text.SimpleDateFormat("dd MMM yyyy HH:mm:ss Z")

	new File(dir).listFiles.
		sortBy(_.lastModified).reverse.
		filter(_.getName().endsWith(".mp4")).
		map(file => file.substring(dir.length + 1)).
		foreach(file => 
		{
			val f = new File(dir,file)
           	val entry = new SyndEntryImpl()
            val ii = file.indexOf('-')
            if (ii != -1)
            {
                val author = file.substring(0, ii)
                var title = file.substring(ii + 1)
                val xx = title.lastIndexOf(".mp4")
                if (xx != -1)
                    title = title.substring(0, xx)
                entry.setTitle(title.replace('_', ' '))
                entry.setAuthor(author.replace('_', ' '))
            }
			val url = "%s/%s/%s".format(Conf.RSS_BASE_URL,channel,file)
            entry.setLink(url);
			entry.setPublishedDate(new java.util.Date(f lastModified))
            val enclosure = new SyndEnclosureImpl();
            enclosure.setUrl(url);
            enclosure.setType("video/mp4");
			enclosure.setLength(f.length())
            val enc = new java.util.ArrayList[SyndEnclosure]();
            enc.add(enclosure);
            entry.setEnclosures(enc);
            entries.add(entry);
		})

	val output = new SyndFeedOutput()
	writeFile("%s/%s/feed.xml".format(Conf.RSS_WEB_DIR, channel), output.outputString(feed))

}

def writeFile(file: String, text : String) : Unit = {
	val fw = new FileWriter(file)
		try{ fw.write(text) }
	finally{ fw.close }
}


case class Stop()
case class Download(entry : NodeSeq, channel : String)

class Downloader extends Actor
{
	start

	def act =
		loop
		{
			react
			{
				case Download(entry, channel) =>
				val link = (entry\"link")(0) \ "@href"
				val fname = List(Conf.BIN_DIR + "/youtube-dl", "--get-filename","-o",Conf.DOWNLOAD_DIR+"/"+channel+"/%(uploader)s-%(stitle)s.%(ext)s",link.toString)!!
				val result = fname trim

				if (!new File(result).exists())
				{
					List(Conf.BIN_DIR + "/youtube-dl", "--quiet","--no-progress","-o",Conf.DOWNLOAD_DIR+"/"+channel+"/%(uploader)s-%(stitle)s.%(ext)s",link.toString)!!

					println("Downloaded " +result)
				}
				else
				{
					println("Skipping downloading of " + result)
				}

				val FileName = String.format("""%s(.*)\..*""",Conf.DOWNLOAD_DIR).r
				val FileName(basename) = result.trim
				val output = Conf.RSS_WEB_DIR + "/" + basename + ".mp4"
				var destFile = new java.io.File(output)
				if (!destFile.exists())
				{
					destFile.getParentFile().mkdirs()
					println("Transcoding %s".format(output))

					List(Conf.BIN_DIR+"/HandBrakeCLI","-Z","iPad","-i",result,"-o",output)!

					List("touch","-c","-r",result, output)!

					updateFeed(channel)
				}
				else
					println("Skipping encoding because dest exists " + destFile)

				case Stop => exit
				case _ => println("Unknown message")
			}
		}
}

implicit def file2str(f:File) : String = f.toString
YoutubeRSS.main(args)
